package info.bukova.isspst.services;

import static ch.lambdaj.Lambda.filter;

import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.validation.ConstraintViolation;
import javax.validation.Validator;

import org.hibernate.HibernateException;
import org.hibernate.NonUniqueResultException;
import org.hibernate.Query;
import org.springframework.transaction.annotation.Transactional;

import info.bukova.isspst.dao.BaseDao;
import info.bukova.isspst.dao.IntegrityException;
import info.bukova.isspst.data.DataModel;
import info.bukova.isspst.filters.Filter;

public abstract class AbstractService<T extends DataModel> implements Service<T> {

	protected BaseDao<T> dao;
	private Validator validator;
	
	public void setDao(BaseDao<T> dao) {
		this.dao = dao;
	}
	
	@Override
	@Transactional
	public void add(T entity) {
		entity.setCreated(new Date());
		dao.add(entity);
	}

	@Override
	@Transactional
	public void update(T entity) {
		entity.setModified(new Date());
		dao.modify(entity);
	}

	@Override
	@Transactional
	public void delete(T entity) {
		try {
			dao.delete(entity);
		} catch (HibernateException e) {
			throw new IntegrityException(e);
		}
	}
	
	@Override
	public void validate(T entity) {
		if (validator == null) {
			entity.setValid(true);
			return;
		}
		
		Set<ConstraintViolation<T>> violations = validator.validate(entity);
		
		if (!violations.isEmpty()) {
			ValidationException ex = new ValidationException();
			for(ConstraintViolation<T> v : violations) {
				String errKye = v.getPropertyPath().toString();
				ex.addMessage(errKye.isEmpty() ? "CLASS_ERR" : errKye, v.getMessage());
			}
			
			throw ex;
		}
		
		entity.setValid(true);
	}
	
	@Override
	public List<T> filterList(List<T> sourceList, Filter<T> filter) {
		return filter(filter.matcher(), sourceList);
	}

	@Override
	@Transactional
	public T getById(int id) {
		
		return dao.getById(id);
	}
	
	@Override
	@Transactional
	public List<T> getAll() {
		return dao.getAll();
	}

	@Override
	@Transactional
	public List<T> execQuery(String query) {
		return dao.execQuery(query);
	}

	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	public T selectSingle(String query) {
		try {
			Query q = dao.getQuery(query);
			return (T) q.uniqueResult();
		} catch (NonUniqueResultException e) {
			return null;
		}
	}

	public void setValidator(Validator validator) {
		this.validator = validator;
	}

}
